// Copyright (C) 2008 Tuma Solutions, LLC
// Process Dashboard - Data Automation Tool for high-maturity processes
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// Additional permissions also apply; see the README-license.txt
// file in the project root directory for more information.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.
//
// The author(s) may be contacted at:
//     processdash@tuma-solutions.com
//     processdash-devel@lists.sourceforge.net

package net.sourceforge.processdash.tool.bridge.report;

import java.io.IOException;
import java.io.OutputStream;
import java.util.List;

import net.sourceforge.processdash.tool.bridge.ResourceCollection;


/**
 * Interface for an object which is able to generate output from the data in a
 * {@link ResourceCollection}.
 */
public interface CollectionReport {

    /** Get the mime type of the data generated by this report */
    public String getContentType();

    /**
     * Produce the output of this report
     * 
     * @param collection
     *                the collection this report should be based upon
     * @param resourceNames
     *                a list of resource names in the collection to include in
     *                the report
     * @param out
     *                the output stream to which the report should be written
     * @throws IOException
     *                 if any problem occurs while producing the report
     */
    public void runReport(ResourceCollection collection,
            List<String> resourceNames, OutputStream out) throws IOException;

}
